@@ -23,11 +23,12 @@
 #include "config.h"
 
 #include "gkr-daemon.h"
+#include "gkr-daemon-util.h"
 
-#include "common/gkr-buffer.h"
-#include "common/gkr-daemon-util.h"
 #include "common/gkr-location.h"
-#include "common/gkr-secure-memory.h"
+
+#include "egg/egg-buffer.h"
+#include "egg/egg-secure-memory.h"
 
 #include "keyrings/gkr-keyring.h"
 #include "keyrings/gkr-keyring-item.h"
@@ -304,41 +305,49 @@ request_keyring_access (GkrKeyringRequest *req, GkrKeyring *keyring)
 	
 	if (app->display_name && app->pathname) {
 		if (is_default) {
+			/* TRANSLATORS: The default keyring is locked */
 			message = g_markup_printf_escaped (_("The application '%s' (%s) wants access to "
 						           "the default keyring, but it is locked"),
 						           app->display_name, app->pathname);
 		} else {
+			/* TRANSLATORS: The keyring '%s' is locked */
 			message = g_markup_printf_escaped (_("The application '%s' (%s) wants access to "
 						           "the keyring '%s', but it is locked"),
 						           app->display_name, app->pathname, keyring_name);
 		}
 	} else if (app->display_name) {
 		if (is_default) {
+			/* TRANSLATORS: The default keyring is locked */
 			message = g_markup_printf_escaped (_("The application '%s' wants access to the "
 						           "default keyring, but it is locked"),
 						           app->display_name);
 		} else {
+			/* TRANSLATORS: The keyring '%s' is locked */
 			message = g_markup_printf_escaped (_("The application '%s' wants access to the "
 						           "keyring '%s', but it is locked"),
 						           app->display_name, keyring_name);
 		} 
 	} else if (app->pathname) {
 		if (is_default) {
+			/* TRANSLATORS: The default keyring is locked */
 			message = g_markup_printf_escaped (_("The application '%s' wants access to the "
 						           "default keyring, but it is locked"),
 						           app->pathname);
 		}
 		else {
+			/* TRANSLATORS: The keyring '%s' is locked */
 			message = g_markup_printf_escaped (_("The application '%s' wants access to the "
 						           "keyring '%s', but it is locked"),
 						           app->pathname, keyring_name);
 		}
 	} else { 
 		if (is_default) {
+			/* TRANSLATORS: The default keyring is locked */
 			message = g_markup_printf_escaped (_("An unknown application wants access to the "
 						           "default keyring, but it is locked"));
 		}
 		else {
+			/* TRANSLATORS: The keyring '%s' is locked */
 			message = g_markup_printf_escaped (_("An unknown application wants access to the "
 						           "keyring '%s', but it is locked"),
 						           keyring_name);
@@ -400,40 +409,48 @@ request_new_keyring_password (GkrKeyringRequest *req, const char *keyring_name,
 
 	if (app->display_name && app->pathname) {
 		if (!is_default) {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' (%s) wants to create a new keyring called '%s'. "
 						           "You have to choose the password you want to use for it."),
 						           app->display_name, app->pathname, keyring_name);
 		} else {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' (%s) wants to create a new default keyring. "
 						           "You have to choose the password you want to use for it."),
 						           app->display_name, app->pathname);
 		} 
 	} else if (app->display_name) {
 		if (!is_default) {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' wants to create a new keyring called '%s'. "
 						           "You have to choose the password you want to use for it."),
 						           app->display_name, keyring_name);
 		} else {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' wants to create a new default keyring. "
 						           "You have to choose the password you want to use for it."),
 						           app->display_name);
 		} 
 	} else if (app->pathname) {
 		if (!is_default) {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' wants to create a new keyring called '%s'. "
 						           "You have to choose the password you want to use for it."),
 						           app->pathname, keyring_name);
 		} else {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("The application '%s' wants to create a new default keyring. "
 						           "You have to choose the password you want to use for it."),
 						           app->pathname);
 		} 
 	} else {
 		if (!is_default) {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("An unknown application wants to create a new keyring called '%s'. "
 						           "You have to choose the password you want to use for it."),
 						           keyring_name);
 		} else {
+			/* TRANSLATORS: The password is for the new keyring */
 			message = g_markup_printf_escaped (_("An unknown application wants to create a new default keyring. "
 						           "You have to choose the password you want to use for it."));
 		} 
@@ -454,7 +471,7 @@ request_new_keyring_password (GkrKeyringRequest *req, const char *keyring_name,
 	ret = ask->response >= GKR_ASK_RESPONSE_ALLOW;
 	if (ret) {
 		g_free (*password);
-		*password = gkr_secure_strdup (ask->typed_password);
+		*password = egg_secure_strdup (ask->typed_password);
 		*volume = ask->location_selected;
 	}
 	
@@ -550,10 +567,10 @@ request_change_keyring_password (GkrKeyringRequest *req, GkrKeyring* keyring,
 	ret = ask->response >= GKR_ASK_RESPONSE_ALLOW;
 	if (ret) {
 		g_free (*password);
-		*password = gkr_secure_strdup (ask->typed_password);
+		*password = egg_secure_strdup (ask->typed_password);
 		
 		g_free (*original);
-		*original = gkr_secure_strdup (ask->original_password);
+		*original = egg_secure_strdup (ask->original_password);
 	}
 	
 	g_object_unref (ask);
@@ -686,14 +703,14 @@ change_keyring_password (GkrKeyring *keyring,  const char *password)
 	if (keyring->locked) {
 		return GNOME_KEYRING_RESULT_DENIED;
 	} else { 
-		keyring->password = gkr_secure_strdup (password);
+		keyring->password = egg_secure_strdup (password);
 		gkr_keyring_save_to_disk (keyring);
 		return GNOME_KEYRING_RESULT_OK;
 	}
 }
 
 static gboolean
-op_lock_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_lock_keyring (EggBuffer *packet, EggBuffer *result,
                  GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -705,10 +722,10 @@ op_lock_keyring (GkrBuffer *packet, GkrBuffer *result,
 
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 	} else {
 		gkr_keyring_lock (keyring);
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	}
 	
 	g_free (keyring_name);
@@ -724,16 +741,16 @@ lock_each_keyring (GkrKeyring* keyring, gpointer unused)
 }
 
 static gboolean
-op_lock_all (GkrBuffer *packet, GkrBuffer *result,
+op_lock_all (EggBuffer *packet, EggBuffer *result,
              GkrKeyringRequest *req)
 {
 	gkr_keyrings_foreach (lock_each_keyring, NULL);
-	gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+	egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	return TRUE;
 }
 
 static gboolean
-op_set_default_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_set_default_keyring (EggBuffer *packet, EggBuffer *result,
                         GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -745,14 +762,14 @@ op_set_default_keyring (GkrBuffer *packet, GkrBuffer *result,
 
 	if (keyring_name == NULL) {
 		gkr_keyrings_set_default (NULL);
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	} else {
 		keyring = gkr_keyrings_find (keyring_name);
 		if (keyring == NULL) {
-			gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+			egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 		} else {
 			gkr_keyrings_set_default (keyring);
-			gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+			egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 		}
 	}
 	
@@ -762,13 +779,13 @@ op_set_default_keyring (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_get_default_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_get_default_keyring (EggBuffer *packet, EggBuffer *result,
                         GkrKeyringRequest *req)
 {
 	GkrKeyring* keyring;
 	char *name;
 	
-	gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+	egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	
 	name = NULL;
 	
@@ -785,17 +802,17 @@ op_get_default_keyring (GkrBuffer *packet, GkrBuffer *result,
 static gboolean
 add_name_to_result (GkrKeyring* keyring, gpointer result)
 {
-	return gkr_proto_add_utf8_string ((GkrBuffer*)result, 
+	return gkr_proto_add_utf8_string ((EggBuffer*)result, 
 	                                  keyring->keyring_name);
 }
 
 static gboolean
-op_list_keyrings (GkrBuffer *packet, GkrBuffer *result,
+op_list_keyrings (EggBuffer *packet, EggBuffer *result,
                   GkrKeyringRequest *req)
 {
-	gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+	egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 
-	gkr_buffer_add_uint32 (result, gkr_keyrings_get_count ());
+	egg_buffer_add_uint32 (result, gkr_keyrings_get_count ());
 	if (!gkr_keyrings_foreach (add_name_to_result, result))
 		return FALSE;
 	
@@ -804,7 +821,7 @@ op_list_keyrings (GkrBuffer *packet, GkrBuffer *result,
 
 
 static gboolean
-op_set_keyring_info (GkrBuffer *packet, GkrBuffer *result,
+op_set_keyring_info (EggBuffer *packet, EggBuffer *result,
                      GkrKeyringRequest *req)
 {
 	char    *keyring_name;
@@ -821,9 +838,9 @@ op_set_keyring_info (GkrBuffer *packet, GkrBuffer *result,
 	
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 	} else {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 		
 		keyring->lock_on_idle = lock_on_idle;
 		keyring->lock_timeout = lock_timeout;
@@ -835,7 +852,7 @@ op_set_keyring_info (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_get_keyring_info (GkrBuffer *packet, GkrBuffer *result,
+op_get_keyring_info (EggBuffer *packet, EggBuffer *result,
                      GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -847,15 +864,15 @@ op_get_keyring_info (GkrBuffer *packet, GkrBuffer *result,
 	
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 	} else {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 		
-		gkr_buffer_add_uint32 (result, keyring->lock_on_idle);
-		gkr_buffer_add_uint32 (result, keyring->lock_timeout);
+		egg_buffer_add_uint32 (result, keyring->lock_on_idle);
+		egg_buffer_add_uint32 (result, keyring->lock_timeout);
 		gkr_proto_add_time (result, keyring->mtime);
 		gkr_proto_add_time (result, keyring->ctime);
-		gkr_buffer_add_uint32 (result, keyring->locked);
+		egg_buffer_add_uint32 (result, keyring->locked);
 	}
 	
 	g_free (keyring_name);
@@ -864,7 +881,7 @@ op_get_keyring_info (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_create_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_create_keyring (EggBuffer *packet, EggBuffer *result,
                    GkrKeyringRequest *req)
 {
 	GQuark volume = GKR_LOCATION_VOLUME_LOCAL;
@@ -881,25 +898,25 @@ op_create_keyring (GkrBuffer *packet, GkrBuffer *result,
 	g_assert (opcode == GNOME_KEYRING_OP_CREATE_KEYRING);
 
 	if (keyring_name == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_BAD_ARGUMENTS);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_BAD_ARGUMENTS);
 		goto out;
 	}
 	
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring != NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_ALREADY_EXISTS);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_ALREADY_EXISTS);
 		goto out;
 	}
 	
 	/* Let user pick password if necessary*/
 	if (!request_new_keyring_password (req, keyring_name, &password, &volume)) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 		goto out;
 	}
 	
 	keyring = gkr_keyring_create (volume, keyring_name, password);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 		goto out;
 	}
 	
@@ -910,17 +927,17 @@ op_create_keyring (GkrBuffer *packet, GkrBuffer *result,
 	g_object_unref (keyring);
 	g_assert (GKR_IS_KEYRING (keyring));
 	
-	gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+	egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	
  out:
 	g_free (keyring_name);
-	gkr_secure_strfree (password);
+	egg_secure_strfree (password);
 
 	return TRUE;
 }
 
 static gboolean
-op_unlock_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_unlock_keyring (EggBuffer *packet, EggBuffer *result,
                    GkrKeyringRequest *req)
 {
 	char *keyring_name, *password;
@@ -938,7 +955,7 @@ op_unlock_keyring (GkrBuffer *packet, GkrBuffer *result,
 	
 	keyring = gkr_keyrings_find (keyring_name);
 	if (!keyring) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 		goto out;
 
 	} 
@@ -946,9 +963,9 @@ op_unlock_keyring (GkrBuffer *packet, GkrBuffer *result,
 	/* User types password */
 	if (password == NULL) {
 		if (request_keyring_access (req, keyring)) 
-			gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+			egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 		else 
-			gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+			egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 			
 	/* Password specified */
 	} else {
@@ -956,19 +973,19 @@ op_unlock_keyring (GkrBuffer *packet, GkrBuffer *result,
 			res = GNOME_KEYRING_RESULT_OK;
 		else
 			res = GNOME_KEYRING_RESULT_DENIED;
-		gkr_buffer_add_uint32 (result, res);
+		egg_buffer_add_uint32 (result, res);
 	} 
 
  out:
 	g_free (keyring_name);
-	gkr_secure_strfree (password);
+	egg_secure_strfree (password);
 
 	return TRUE;
 }
 
 
 static gboolean
-op_delete_keyring (GkrBuffer *packet, GkrBuffer *result,
+op_delete_keyring (EggBuffer *packet, EggBuffer *result,
                    GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -995,7 +1012,7 @@ op_delete_keyring (GkrBuffer *packet, GkrBuffer *result,
 		}
 	}
 	
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	g_free (keyring_name);
 	
 	if (res == GNOME_KEYRING_RESULT_OK)
@@ -1005,7 +1022,7 @@ op_delete_keyring (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_change_keyring_password (GkrBuffer *packet, GkrBuffer *result,
+op_change_keyring_password (EggBuffer *packet, EggBuffer *result,
                             GkrKeyringRequest *req)
 {
 	char *keyring_name, *original, *password;
@@ -1022,41 +1039,41 @@ op_change_keyring_password (GkrBuffer *packet, GkrBuffer *result,
 	g_assert (opcode == GNOME_KEYRING_OP_CHANGE_KEYRING_PASSWORD);
 	
 	if (keyring_name == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_BAD_ARGUMENTS);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_BAD_ARGUMENTS);
 		goto out;
 	}
 
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
 		goto out;
 	}
 
 	/* Prompt for any missing passwords */
 	if (!request_change_keyring_password (req, keyring, &original, &password)) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 		goto out;
 	}
 	
 	gkr_keyring_lock (keyring);
 	
 	if (!gkr_keyring_unlock (keyring, original)) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 		goto out;
 	}
 	
-	gkr_buffer_add_uint32 (result, change_keyring_password (keyring, password));
+	egg_buffer_add_uint32 (result, change_keyring_password (keyring, password));
 	
  out:
 	g_free (keyring_name);
-	gkr_secure_strfree (original);
-	gkr_secure_strfree (password);
+	egg_secure_strfree (original);
+	egg_secure_strfree (password);
 	
 	return TRUE;
 }
 
 static gboolean
-op_list_items (GkrBuffer *packet, GkrBuffer *result,
+op_list_items (EggBuffer *packet, EggBuffer *result,
                GkrKeyringRequest *req)
 {
 	GkrKeyring *keyring;
@@ -1070,16 +1087,16 @@ op_list_items (GkrBuffer *packet, GkrBuffer *result,
 	
 	keyring = gkr_keyrings_find (keyring_name);
 	if (keyring == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
-		gkr_buffer_add_uint32 (result, 0);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+		egg_buffer_add_uint32 (result, 0);
 		
 	} else if (!request_keyring_access (req, keyring)) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
-		gkr_buffer_add_uint32 (result, 0);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, 0);
 	
 	} else {
 
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 			
 		items = NULL;
 		for (l = keyring->items; l != NULL; l = l->next) {
@@ -1089,10 +1106,10 @@ op_list_items (GkrBuffer *packet, GkrBuffer *result,
 		items = g_list_reverse (items);
 
 		/* Send the results */
-		gkr_buffer_add_uint32 (result, g_list_length (items));
+		egg_buffer_add_uint32 (result, g_list_length (items));
 		for (l = items; l != NULL; l = l->next) {
 			item = l->data;
-			gkr_buffer_add_uint32 (result, item->id);
+			egg_buffer_add_uint32 (result, item->id);
 		}
 
 		g_list_free (items);
@@ -1104,7 +1121,7 @@ op_list_items (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_create_item (GkrBuffer *packet, GkrBuffer *result,
+op_create_item (EggBuffer *packet, EggBuffer *result,
 		GkrKeyringRequest *req)
 {
 	char *keyring_name, *display_name, *secret;
@@ -1182,8 +1199,8 @@ op_create_item (GkrBuffer *packet, GkrBuffer *result,
 
 	g_free (item->display_name);
 	item->display_name = g_strdup (display_name);
-	gkr_secure_strfree (item->secret);
-	item->secret = gkr_secure_strdup (secret);
+	egg_secure_strfree (item->secret);
+	item->secret = egg_secure_strdup (secret);
 	gnome_keyring_attribute_list_free (item->attributes);
 	item->attributes = gnome_keyring_attribute_list_copy (attributes);
 	
@@ -1198,17 +1215,17 @@ op_create_item (GkrBuffer *packet, GkrBuffer *result,
  out:	
 	g_free (keyring_name);
 	g_free (display_name);
-	gkr_secure_strfree (secret);
+	egg_secure_strfree (secret);
 	gnome_keyring_attribute_list_free (hashed);
 	gnome_keyring_attribute_list_free (attributes);
 	
-	gkr_buffer_add_uint32 (result, res);
-	gkr_buffer_add_uint32 (result, id);
+	egg_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, id);
 	return TRUE;
 }
 
 static gboolean
-op_delete_item (GkrBuffer *packet, GkrBuffer *result,
+op_delete_item (EggBuffer *packet, EggBuffer *result,
                 GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1233,7 +1250,7 @@ op_delete_item (GkrBuffer *packet, GkrBuffer *result,
 	                                      TRUE, 
 	                                      &item);
 	                                      
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
 		if (item->keyring) {
 			keyring = item->keyring;
@@ -1247,7 +1264,7 @@ op_delete_item (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_get_item_info (GkrBuffer *packet, GkrBuffer *result,
+op_get_item_info (EggBuffer *packet, EggBuffer *result,
                   GkrKeyringRequest *req)
 {
 	char *keyring_name, *secret;
@@ -1270,9 +1287,9 @@ op_get_item_info (GkrBuffer *packet, GkrBuffer *result,
 	                                      (flags & GNOME_KEYRING_ITEM_INFO_SECRET) == GNOME_KEYRING_ITEM_INFO_SECRET, 
 	                                      &item);
 
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
-		gkr_buffer_add_uint32 (result, item->type);
+		egg_buffer_add_uint32 (result, item->type);
 		if (!gkr_proto_add_utf8_string (result, item->display_name))
 			ret = FALSE;
 
@@ -1294,7 +1311,7 @@ op_get_item_info (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_get_item_attributes (GkrBuffer *packet, GkrBuffer *result,
+op_get_item_attributes (EggBuffer *packet, EggBuffer *result,
                         GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1318,7 +1335,7 @@ op_get_item_attributes (GkrBuffer *packet, GkrBuffer *result,
 	                                      FALSE, 
 	                                      &item);
 
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
 		if (!gkr_proto_add_attribute_list (result, item->attributes))
 			ret = FALSE;
@@ -1329,7 +1346,7 @@ op_get_item_attributes (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_get_item_acl (GkrBuffer *packet, GkrBuffer *result,
+op_get_item_acl (EggBuffer *packet, EggBuffer *result,
                  GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1353,7 +1370,7 @@ op_get_item_acl (GkrBuffer *packet, GkrBuffer *result,
 	                                      FALSE, 
 	                                      &item);
 
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
 		if (!gkr_proto_add_acl (result, item->acl)) 
 			ret = FALSE;
@@ -1364,7 +1381,7 @@ op_get_item_acl (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_set_item_acl (GkrBuffer *packet, GkrBuffer *result,
+op_set_item_acl (EggBuffer *packet, EggBuffer *result,
                  GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1395,7 +1412,7 @@ op_set_item_acl (GkrBuffer *packet, GkrBuffer *result,
 			gkr_keyring_save_to_disk (item->keyring);
 	}
 	
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 
 	gnome_keyring_acl_free (acl);
 	g_free (keyring_name);
@@ -1403,7 +1420,7 @@ op_set_item_acl (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_set_item_info (GkrBuffer *packet, GkrBuffer *result,
+op_set_item_info (EggBuffer *packet, EggBuffer *result,
                   GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1428,7 +1445,7 @@ op_set_item_info (GkrBuffer *packet, GkrBuffer *result,
 	                                      TRUE, 
 	                                      &item);
 
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
 		if ((type & GNOME_KEYRING_ITEM_TYPE_MASK) != GNOME_KEYRING_ITEM_NO_TYPE) {
 			item->type = type;
@@ -1438,8 +1455,8 @@ op_set_item_info (GkrBuffer *packet, GkrBuffer *result,
 			item->display_name = g_strdup (item_name);
 		}
 		if (secret != NULL) {
-			gkr_secure_strfree (item->secret);
-			item->secret = gkr_secure_strdup (secret);
+			egg_secure_strfree (item->secret);
+			item->secret = egg_secure_strdup (secret);
 		}
 
 		if (item->keyring)
@@ -1448,12 +1465,12 @@ op_set_item_info (GkrBuffer *packet, GkrBuffer *result,
 
 	g_free (keyring_name);
 	g_free (item_name);
-	gkr_secure_strfree (secret);
+	egg_secure_strfree (secret);
 	return TRUE;
 }
 
 static gboolean
-op_set_daemon_display (GkrBuffer *packet, GkrBuffer *result,
+op_set_daemon_display (EggBuffer *packet, EggBuffer *result,
                        GkrKeyringRequest *req)
 {
 	char *display;
@@ -1463,10 +1480,10 @@ op_set_daemon_display (GkrBuffer *packet, GkrBuffer *result,
 		return FALSE;
 
 	if (display == NULL) {
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 	} else {
 		g_setenv ("DISPLAY", display, FALSE);
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 	}
 
 	g_free (display);
@@ -1474,7 +1491,7 @@ op_set_daemon_display (GkrBuffer *packet, GkrBuffer *result,
 }
 
 static gboolean
-op_set_item_attributes (GkrBuffer *packet, GkrBuffer *result,
+op_set_item_attributes (EggBuffer *packet, EggBuffer *result,
                         GkrKeyringRequest *req)
 {
 	char *keyring_name;
@@ -1497,7 +1514,7 @@ op_set_item_attributes (GkrBuffer *packet, GkrBuffer *result,
 	                                      TRUE, 
 	                                      &item);
 
-	gkr_buffer_add_uint32 (result, res);
+	egg_buffer_add_uint32 (result, res);
 	if (res == GNOME_KEYRING_RESULT_OK) {
 		gnome_keyring_attribute_list_free (item->attributes);
 		item->attributes = gnome_keyring_attribute_list_copy (attributes);
@@ -1614,7 +1631,7 @@ unref_object (gpointer obj, gpointer data)
 }
 
 static gboolean
-op_find (GkrBuffer *packet, GkrBuffer *result, GkrKeyringRequest *req)
+op_find (EggBuffer *packet, EggBuffer *result, GkrKeyringRequest *req)
 {
 	FindContext ctx;
 	GList *l;
@@ -1639,15 +1656,15 @@ op_find (GkrBuffer *packet, GkrBuffer *result, GkrKeyringRequest *req)
 
 	/* No items given access to */
 	if (ctx.nfound > 0 && ctx.items == NULL)
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_DENIED);
 		
 	/* Zero items matched  */
 	else if (ctx.nfound == 0)
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_MATCH);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_NO_MATCH);
 
 	/* More than one item found and given access to */
 	else	
-		gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 
 	ctx.items = g_list_sort_with_data (ctx.items, sort_found, ctx.attributes);
 	
@@ -1664,7 +1681,7 @@ op_find (GkrBuffer *packet, GkrBuffer *result, GkrKeyringRequest *req)
 				break;
 			}
 	    	        
-			gkr_buffer_add_uint32 (result, item->id);
+			egg_buffer_add_uint32 (result, item->id);
 			
 			if (!gkr_proto_add_utf8_secret (result, item->secret) ||
 			    !gkr_proto_add_attribute_list (result, item->attributes)) {
@@ -1684,11 +1701,12 @@ op_find (GkrBuffer *packet, GkrBuffer *result, GkrKeyringRequest *req)
 }
 
 static gboolean
-op_prepare_daemon_environment (GkrBuffer *packet, GkrBuffer *result, GkrKeyringRequest *req)
+op_prepare_daemon_environment (EggBuffer *packet, EggBuffer *result, GkrKeyringRequest *req)
 {
 	const gchar **daemonenv;
 	gchar **environment, **e;
 	gchar *x;
+	gint i;
 
 	if (!gkr_proto_decode_prepare_environment (packet, &environment))
 		return FALSE;
@@ -1699,26 +1717,32 @@ op_prepare_daemon_environment (GkrBuffer *packet, GkrBuffer *result, GkrKeyringR
 		if (x) {
 			*(x++) = 0;
 			
-			/* We're only interested in these guys */
-			if (g_str_equal (*e, "DISPLAY")) 
-				g_setenv ("DISPLAY", x, FALSE);
-			else if (g_str_equal (*e, "DBUS_SESSION_BUS_ADDRESS"))
-				g_setenv ("DBUS_SESSION_BUS_ADDRESS", x, FALSE);
-			else if (g_str_equal (*e, "XAUTHORITY"))
-				g_setenv ("XAUTHORITY", x, FALSE);
-			else if (g_str_equal (*e, "XDG_SESSION_COOKIE"))
-				g_setenv ("XDG_SESSION_COOKIE", x, FALSE);
+			/* We're only interested in these environment variables */
+			for (i = 0; GNOME_KEYRING_IN_ENVIRONMENT[i] != NULL; ++i) {
+				if (g_str_equal (*e, GNOME_KEYRING_IN_ENVIRONMENT[i]))
+				{
+					g_setenv (*e, x, FALSE);
+					break;
+				}
+			}
 		}
 	}
 	
 	g_strfreev (environment);
+	
+	/* 
+	 * We've now definitely received everything we need to run. Ask
+	 * the daemon to complete the initialization. 
+	 */
+	gkr_daemon_complete_initialization();
 
-	gkr_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+	egg_buffer_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 
+	/* These are the environment variables that the daemon setup */
 	daemonenv = gkr_daemon_util_get_environment ();
 	g_return_val_if_fail (daemonenv, FALSE);
 	
-	gkr_buffer_add_stringv (result, daemonenv);
+	egg_buffer_add_stringv (result, daemonenv);
 	return TRUE;
 }
 
